from typing import List, Optional
from datetime import datetime, timedelta

from tea import timestamp as ts

from traktor import errors
from traktor.models import User, Entry, Report
from traktor.engine.task_mixin import TaskMixin


class TimerMixin(TaskMixin):
    # Timer

    @classmethod
    def timer_start(
        cls, user: User, project_id: str, task_id: Optional[str] = None
    ) -> Entry:
        # First see if there are running timers
        entry = Entry.objects.filter(
            task__project__user=user, end_time=None
        ).first()
        if entry is not None:
            raise errors.TimerAlreadyRunning(
                project_id=entry.task.project.slug, task_id=entry.task.slug
            )

        if task_id is None:
            task = cls.task_get_default(user=user, project_id=project_id)
            if task is None:
                raise errors.NoDefaultTask(
                    user=user.username, project_id=project_id
                )
        else:
            task = cls.task_get(
                user=user, project_id=project_id, task_id=task_id
            )

        return Entry.objects.create(task=task)

    @staticmethod
    def timer_stop(user: User) -> Entry:
        entry = Entry.objects.filter(
            task__project__user=user, end_time=None
        ).first()
        if entry is None:
            raise errors.TimerIsNotRunning()

        entry.stop()
        entry.save()
        return entry

    @staticmethod
    def timer_status(user: User) -> Optional[Entry]:
        entry = Entry.objects.filter(
            task__project__user=user, end_time=None
        ).first()
        if entry is None:
            raise errors.TimerIsNotRunning()
        return entry

    @staticmethod
    def _make_report(entries: List[Entry]) -> List[Report]:
        reports = {}
        for entry in entries:
            report = Report(
                user=entry.task.project.user.username,
                project=entry.task.project.name,
                task=entry.task.name,
                duration=(
                    entry.duration
                    or int((ts.now() - entry.start_time).total_seconds())
                ),
            )
            if report.key in reports:
                reports[report.key].duration += report.duration
            else:
                reports[report.key] = report
        return list(reports.values())

    @classmethod
    def timer_today(cls, user: User):
        now = ts.now()
        today = ts.make_aware(datetime(now.year, now.month, now.day))
        return cls._make_report(
            Entry.objects.filter(
                task__project__user=user, start_time__gt=today
            )
        )

    @classmethod
    def timer_report(cls, user: User, days: int = 0) -> List[Report]:
        if days == 0:
            entries = Entry.objects.filter(task__project__user=user)
        else:
            dt = ts.now() - timedelta(days=days)
            since = ts.make_aware(datetime(dt.year, dt.month, dt.day))
            entries = Entry.objects.filter(
                task__project__user=user, start_time__gt=since
            )
        return cls._make_report(entries)



import typer
from pathlib import Path
from typing import Optional

from tea_console.console import command
from tea_console.enums import ConsoleFormat
from tea_console.commands.config import app as config_app

from traktor.config import config
from traktor.commands.db import app as db_app
from traktor.commands.project import app as project_app
from traktor.commands.task import app as task_app
from traktor.commands import timer
from traktor.models import Entry, Report


app = typer.Typer(name="traktor", help="Personal time tracking.")


# Add tea subcommands
app.add_typer(config_app)

# Add traktor subcommands
app.add_typer(db_app)
app.add_typer(project_app)
app.add_typer(task_app)


# Add timer commands as top level
command(app)(timer.status)
command(app, model=Entry)(timer.start)
command(app, model=Entry)(timer.stop)
command(app, model=Report)(timer.today)
command(app, model=Report)(timer.report)


@app.callback()
def callback(
    config_path: Path = typer.Option(
        default=None,
        help="Path to the configuration.",
        exists=True,
        file_okay=True,
        dir_okay=False,
        writable=True,
        readable=True,
        resolve_path=True,
    ),
    fmt: ConsoleFormat = typer.Option(
        config.format.value, "--format", help="Output format"
    ),
    user: Optional[str] = typer.Option(
        None, metavar="username", help="Optionally select different user."
    ),
):
    if config_path is not None:
        config.config_path = config_path

    if config.format != fmt:
        config.format = fmt

    if user is not None:
        config.set_user(user)


@app.command(hidden=True)
def shell():
    """Run IPython shell with loaded configuration and models."""
    try:
        from IPython import embed
        from traktor.config import config
        from traktor.engine import engine
        from traktor.models import User, Project, Task, Entry

        embed(
            user_ns={
                "config": config,
                "engine": engine,
                "User": User,
                "Project": Project,
                "Task": Task,
                "Entry": Entry,
            },
            colors="neutral",
        )
    except ImportError:
        typer.secho("IPython is not installed", color=typer.colors.RED)



from django.db import models

from tea import timestamp as ts
from tea_console.table import Column
from tea_django.models import UUIDBaseModel
from tea_django.models.mixins import TimestampedMixin, TimerMixin

from traktor.models.task import Task


class EntryManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().select_related("task", "task__project")


class Entry(UUIDBaseModel, TimestampedMixin, TimerMixin):
    HEADERS = [
        Column(title="Project", path="task.project.name"),
        Column(title="Task", path="task.name"),
        Column(
            title="Start Time",
            path=lambda o: ts.to_localtime_str(o.start_time),
        ),
        Column(
            title="End Time", path=lambda o: ts.to_localtime_str(o.end_time),
        ),
        Column(title="Duration", path="running_time"),
    ]
    task = models.ForeignKey(
        Task, null=True, blank=True, on_delete=models.SET_NULL
    )
    description = models.CharField(
        max_length=1023, null=False, blank=True, default=""
    )
    notes = models.TextField(null=False, blank=True, default="")

    def __str__(self):
        return (
            f"Entry(project={self.task.project.slug}, "
            f"task={self.task.slug if self.task is not None else None}, "
            f"running_time={self.running_time})"
        )

    __repr__ = __str__

    def to_dict(self) -> dict:
        d = super().to_dict()
        d.update(
            {
                "project": self.task.project.slug,
                "task": self.task.slug,
                "running_time": self.running_time,
            }
        )
        return d

    objects = EntryManager()

    class Meta:
        app_label = "traktor"



from pathlib import Path
from typing import Optional

from tea_django.config import ConfigField, Config as TeaConfig


class Config(TeaConfig):
    ENTRIES = {
        **TeaConfig.ENTRIES,
        "db_engine": ConfigField(section="database", option="engine"),
        "db_name": ConfigField(section="database", option="name"),
        "db_user": ConfigField(section="database", option="user"),
        "db_password": ConfigField(section="database", option="password"),
        "db_host": ConfigField(section="database", option="host"),
        "db_port": ConfigField(section="database", option="port", type=int),
    }

    def __init__(self, config_file: Optional[str] = None):
        # Path to the configuration file
        self.config_dir = (Path("~").expanduser() / ".traktor").absolute()

        # Directory structure
        self.module_dir = Path(__file__).parent.absolute()

        # Database
        self.db_engine = "sqlite3"
        self.db_name = f"{self.config_dir}/traktor.db"
        self.db_user = None
        self.db_password = None
        self.db_host = None
        self.db_port = None

        super().__init__(
            config_file=config_file or (self.config_dir / "traktor.ini")
        )


config = Config()



import os
import json
from pathlib import Path

from tea import serde
from django.core.management import execute_from_command_line

from traktor.config import config
from traktor.models import Project, Task, Entry


class DBEngine:
    @staticmethod
    def ensure():
        if config.db_engine == "sqlite3":
            os.makedirs(os.path.dirname(config.db_name), exist_ok=True)
        execute_from_command_line(["traktor", "migrate", "-v", "0"])

    @staticmethod
    def export(path: Path):
        export = {
            "projects": [
                project.column_dict() for project in Project.objects.all()
            ],
            "tasks": [task.column_dict() for task in Task.objects.all()],
            "entries": [entry.column_dict() for entry in Entry.objects.all()],
        }
        os.makedirs(path.parent, exist_ok=True)
        path.write_text(serde.json_dumps(export, indent=4), encoding="utf-8")

    @staticmethod
    def load(path: Path):
        data = json.loads(path.read_text(encoding="utf-8"))

        for d in data["projects"]:
            project = Project.from_dict(d)
            project.save()

        for d in data["tasks"]:
            task = Task.from_dict(d)
            task.save()

        for d in data["entries"]:
            entry = Entry.from_dict(d)
            entry.save()



from django.db import models

from tea_console.table import Column
from tea_django.models import UUIDBaseModel
from tea_django.models.mixins import (
    ColoredMixin,
    NonUniqueSlugMixin,
    TimestampedMixin,
)
from traktor.models.user import User


class ProjectManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().select_related("user")


class Project(
    UUIDBaseModel, ColoredMixin, NonUniqueSlugMixin, TimestampedMixin
):
    HEADERS = [
        Column(title="ID", path="slug"),
        Column(title="Name", path="rich_name"),
    ]
    user = models.ForeignKey(
        User,
        null=False,
        blank=False,
        default=User.get_superuser_id,
        on_delete=models.CASCADE,
    )
    name = models.CharField(max_length=255)

    @property
    def rich_name(self) -> str:
        return self.rich(self.name)

    def __str__(self):
        return f"{self.name}"

    __repr__ = __str__

    objects = ProjectManager()

    class Meta:
        app_label = "traktor"
        unique_together = ("user", "slug")



__all__ = ["User", "Project", "Task", "Entry", "Report", "signals"]

from traktor.models.user import User
from traktor.models.project import Project
from traktor.models.task import Task
from traktor.models.entry import Entry
from traktor.models.report import Report
from traktor.models import signals



from django.contrib.auth.models import AbstractUser


class User(AbstractUser):
    """Traktor custom user model."""

    @classmethod
    def get_superuser(cls) -> "User":
        user = cls.objects.filter(is_superuser=True).first()
        if user is not None:
            return user

        return cls.objects.create_superuser(
            username="admin", email="admin@traktor.app"
        )

    @classmethod
    def get_superuser_id(cls) -> int:
        return cls.get_superuser().id

    class Meta(AbstractUser.Meta):
        app_label = "traktor"



from typing import List, Optional

from tea_django import errors

from traktor.models import User, Project, Task
from traktor.engine.project_mixin import ProjectMixin


class TaskMixin(ProjectMixin):
    @classmethod
    def task_list(cls, user: User, project_id: Optional[str]) -> List[Task]:
        """List all tasks in a project.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
        """
        if project_id is None:
            query = Task.objects.filter(project__user=user)
        else:
            query = Task.objects.filter(
                project__user=user, project__slug=project_id
            )

        return list(query)

    @classmethod
    def task_get(cls, user: User, project_id: str, task_id: str) -> Task:
        """Get a specific task.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
            task_id (str): Task slug.
        """
        try:
            return Task.get_by_slug(
                slug=task_id, project__user=user, project__slug=project_id,
            )
        except Task.DoesNotExist:
            raise errors.ObjectNotFound(
                model=Task,
                query={
                    "user": user.username,
                    "project_id": project_id,
                    "task_id": task_id,
                },
            )

    @classmethod
    def task_get_default(cls, user: User, project_id: str) -> Optional[Task]:
        """Get default task for a project.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
        """
        try:
            return Task.objects.get(
                project__user=user, project__slug=project_id, default=True
            )
        except Task.DoesNotExist:
            raise errors.ObjectNotFound(
                model=Task,
                query={
                    "user": user.username,
                    "project_id": project_id,
                    "default": True,
                },
            )

    @staticmethod
    def __set_default_task(task: Task, default: bool):
        if default:
            # If the default value for a new task or task update is set to
            # `True` we must first find the previous default task and set it
            # default to `False`.
            try:
                old_default = Task.objects.get(
                    project=task.project, default=True
                )
                # If it's not the same task
                if old_default.pk != task.pk:
                    old_default.default = False
                    old_default.save()
            except Task.DoesNotExist:
                # All OK!
                pass

            # Now set the new task to be default
            task.default = True
            task.save()
        else:
            # It's just a non default task
            task.default = False
            task.save()

    @classmethod
    def task_create(
        cls,
        user: User,
        project_id: str,
        name: str,
        color: Optional[str] = None,
        default: Optional[bool] = None,
    ) -> Task:
        """Create a task.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
            name (str): Task name.
            color (str, optional): Task color.
            default (bool, optional): Is this a default task for this project.
        """
        project = Project.get_by_slug(slug=project_id, user=user)
        try:
            Task.get_by_slug_field(value=name, project__id=project.id)
            raise errors.ObjectAlreadyExists(
                Task,
                query={
                    "user": user.username,
                    "project_id": project.slug,
                    "name": name,
                },
            )
        except Task.DoesNotExist:
            task = Task.objects.create(
                project=project,
                name=name,
                color=color or Task.color.field.default,
            )
            task.save()

        if default is not None:
            cls.__set_default_task(task=task, default=default)

        return task

    @classmethod
    def task_update(
        cls,
        user: User,
        project_id: str,
        task_id: str,
        name: Optional[str],
        color: Optional[str],
        default: Optional[bool],
    ) -> Task:
        """Update a task.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
            task_id (str): Task slug.
            name (str, optional): Task name.
            color (str, optional): Task color.
            default (bool, optional): Is this a default task for this project.
        """
        task = cls.task_get(user=user, project_id=project_id, task_id=task_id)
        # Change name
        if name is not None:
            task.name = name
        # Change color
        if color is not None:
            task.color = color
        # Change default
        if default is not None:
            cls.__set_default_task(task=task, default=default)
        task.save()
        return task

    @classmethod
    def task_delete(cls, user: User, project_id: str, task_id: str):
        """Delete a task.

        Args:
            user (User): Selected user.
            project_id (str): Project slug.
            task_id (str): Task slug.
        """
        task = cls.task_get(user=user, project_id=project_id, task_id=task_id)
        task.delete()



from django.db import models

from tea_console.table import Column
from tea_django.models import UUIDBaseModel
from tea_django.models.mixins import (
    ColoredMixin,
    NonUniqueSlugMixin,
    TimestampedMixin,
)

from traktor.models.project import Project


class TaskManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().select_related("project")


class Task(UUIDBaseModel, ColoredMixin, NonUniqueSlugMixin, TimestampedMixin):
    HEADERS = [
        Column(title="Project ID", path="project.slug"),
        Column(title="Task ID", path="slug"),
        Column(title="Name", path="rich_name"),
        Column(title="Default", path="default"),
    ]

    project = models.ForeignKey(
        Project, blank=False, null=False, on_delete=models.CASCADE
    )
    name = models.CharField(max_length=255, blank=False, null=False)
    default = models.BooleanField(default=False, blank=False, null=False)

    @property
    def rich_name(self) -> str:
        return self.rich(self.name)

    def __str__(self):
        return f"{self.project.name} / {self.name})"

    __repr__ = __str__

    objects = TaskManager()

    class Meta:
        app_label = "traktor"
        unique_together = [["project", "slug"]]



from typing import List, Optional

from tea_django import errors

from traktor.models import User, Project


class ProjectMixin:
    @staticmethod
    def project_list(user: User) -> List[Project]:
        return list(Project.objects.filter(user=user))

    @staticmethod
    def project_get(user: User, project_id: str) -> Project:
        try:
            return Project.get_by_slug(slug=project_id, user=user)
        except Project.DoesNotExist:
            raise errors.ObjectNotFound(
                model=Project,
                query={"user": user.username, "project_id": project_id},
            )

    @classmethod
    def project_create(
        cls, user: User, name: str, color: Optional[str] = None
    ) -> Project:
        try:
            Project.get_by_slug_field(value=name, user=user)
            raise errors.ObjectAlreadyExists(
                model=Project, query={"user": user.username, "name": name}
            )
        except Project.DoesNotExist:
            return Project.objects.create(
                user=user,
                name=name,
                color=color or Project.color.field.default,
            )

    @classmethod
    def project_update(
        cls,
        user: User,
        project_id: str,
        name: Optional[str],
        color: Optional[str],
    ) -> Project:
        project = cls.project_get(user=user, project_id=project_id)
        # Change name
        if name is not None:
            project.name = name
        # Change color
        if color is not None:
            project.color = color
        project.save()
        return project

    @classmethod
    def project_delete(cls, user: User, project_id: str):
        project = cls.project_get(user=user, project_id=project_id)
        project.delete()



# Generated by Django 3.1 on 2020-08-28 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("traktor", "0005_add_user_to_project"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project", name="slug", field=models.SlugField(),
        ),
        migrations.AlterUniqueTogether(
            name="project", unique_together={("user", "slug")},
        ),
    ]



# Generated by Django 3.1 on 2020-08-20 17:37

from django.db import migrations, models
import django.utils.timezone
import tea_django.models.mixins.colored


class Migration(migrations.Migration):

    dependencies = [
        ("traktor", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="entry",
            name="description",
            field=models.CharField(blank=True, default="", max_length=1023),
        ),
        migrations.AlterField(
            model_name="entry",
            name="notes",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="entry",
            name="start_time",
            field=models.DateTimeField(
                blank=True, default=django.utils.timezone.now
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="color",
            field=tea_django.models.mixins.colored.ColorField(
                default="#FFFFFF", max_length=7
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="color",
            field=tea_django.models.mixins.colored.ColorField(
                default="#FFFFFF", max_length=7
            ),
        ),
    ]



# Generated by Django 3.1 on 2020-08-17 17:08

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("slug", models.SlugField(unique=True)),
                ("color", models.CharField(default="#000000", max_length=7)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name="Task",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="traktor.project",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("slug", models.SlugField()),
                ("color", models.CharField(default="#000000", max_length=7)),
                ("default", models.BooleanField(default=False)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
            ],
            options={"unique_together": {("project", "slug")}},
        ),
        migrations.CreateModel(
            name="Entry",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="traktor.project",
                    ),
                ),
                (
                    "task",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="traktor.task",
                    ),
                ),
                ("description", models.CharField(default="", max_length=1023)),
                ("notes", models.TextField(default="")),
                ("start_time", models.DateTimeField(auto_now_add=True)),
                (
                    "end_time",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                ("duration", models.BigIntegerField(default=0)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("updated_on", models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from traktor.models import User, Project, Task, Entry


def name(s: str):
    def decorator(func):
        func.short_description = s
        return func

    return decorator


class ColoredMixin:
    @name("Color")
    def colored_color(self, obj):
        return obj.html(obj.color)


admin.register(User)(UserAdmin)


@admin.register(Project)
class ProjectAdmin(admin.ModelAdmin, ColoredMixin):
    list_display = ["name", "user", "colored_color"]
    list_filter = ["user__username"]
    ordering = ["user__username", "name"]
    search_fields = ["user__username", "name"]
    readonly_fields = ("user", "slug", "created_on", "updated_on")
    fieldsets = (
        (None, {"fields": ("user", "name", "slug", "color")}),
        ("Timestamps", {"fields": ("created_on", "updated_on")}),
    )


@admin.register(Task)
class TaskAdmin(admin.ModelAdmin, ColoredMixin):
    list_display = ["name", "project_name", "user", "colored_color", "default"]
    list_filter = ["project__user__username"]
    ordering = ["project__user__username", "project__name", "name"]
    search_fields = ["project__user__username", "project__name", "name"]
    readonly_fields = ("user", "slug", "created_on", "updated_on")
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "user",
                    "project",
                    "name",
                    "slug",
                    "default",
                    "color",
                )
            },
        ),
        ("Timestamps", {"fields": ("created_on", "updated_on")}),
    )

    @name("User")
    def user(self, obj):
        return obj.project.user

    @name("Project")
    def project_name(self, obj):
        return obj.project.name


@admin.register(Entry)
class EntryAdmin(admin.ModelAdmin):
    list_display = (
        "task_name",
        "project_name",
        "user",
        "running_time",
        "start_time",
        "end_time",
    )
    list_filter = ["task__project__user__username"]
    ordering = ["-start_time", "task__project__user__username"]
    search_fields = [
        "task__project__user__username",
        "task__project__name",
        "task__name",
    ]
    readonly_fields = (
        "user",
        "project_name",
        "running_time",
        "created_on",
        "updated_on",
    )
    fieldsets = (
        (
            None,
            {
                "fields": (
                    "user",
                    "project_name",
                    "task",
                    "description",
                    "notes",
                )
            },
        ),
        ("Timer", {"fields": ("start_time", "end_time", "running_time")}),
        ("Timestamps", {"fields": ("created_on", "updated_on")}),
    )

    @name("User")
    def user(self, obj):
        return obj.task.project.user

    @name("Project")
    def project_name(self, obj):
        return obj.task.project.name

    @name("Task")
    def task_name(self, obj):
        return obj.task.name



"""
Django settings for traktor project.

Generated by 'django-admin startproject' using Django 3.1.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

from pathlib import Path

from traktor.config import config


# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve(strict=True).parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = config.secret_key

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config.debug


# Application definition

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "tea_django",
    "traktor",
]


# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": f"django.db.backends.{config.db_engine}",
        "NAME": config.db_name,
        "USER": config.db_user,
        "PASSWORD": config.db_password,
        "HOST": config.db_host,
        "PORT": config.db_port,
    }
}


AUTH_USER_MODEL = "traktor.User"

# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation"
        ".UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation"
        ".MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation"
        ".CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation"
        ".NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = config.timezone.zone

USE_I18N = True

USE_L10N = True

USE_TZ = True



# Generated by Django 3.1 on 2020-08-23 17:58

from django.db import migrations, models
import tea.timestamp


class Migration(migrations.Migration):

    dependencies = [
        ("traktor", "0003_user"),
    ]

    operations = [
        migrations.AlterField(
            model_name="entry",
            name="start_time",
            field=models.DateTimeField(blank=True, default=tea.timestamp.now),
        ),
    ]



from pathlib import Path

import typer
from tea_console.console import command

from traktor.engine import engine


app = typer.Typer(name="db", help="Database export/import.")


@command(app, name="export")
def export(path: Path):
    """Export database to JSON document."""
    engine.db.export(path=path)


@command(app, name="import")
def load(path: Path):
    """Import database export from JSON document."""
    engine.db.load(path=path)



from typing import Optional

import typer
from tea_console.console import command

from traktor.config import config
from traktor.engine import engine
from traktor.models import User, Task


app = typer.Typer(name="task", help="Task commands.")


# Make sure that the database exists and it's migrated to the latest version
app.callback()(engine.db.ensure)


@command(app, model=Task, name="list")
def list_tasks(project: Optional[str] = typer.Argument(None)):
    """List all tasks."""
    user = User.objects.get(username=config.selected_user)
    return engine.task_list(user=user, project_id=project)


@command(app, model=Task)
def add(
    project: str,
    name: str,
    color: Optional[str] = None,
    default: Optional[bool] = None,
):
    """Create a task."""
    user = User.objects.get(username=config.selected_user)
    return engine.task_create(
        user=user, project_id=project, name=name, color=color, default=default,
    )


@command(app, model=Task)
def update(
    project: str,
    task_id: str,
    name: Optional[str] = typer.Option(None, help="New task name."),
    color: Optional[str] = typer.Option(None, help="New task color"),
    default: Optional[bool] = typer.Option(
        None, help="Is this a default task."
    ),
):
    """Update a task."""
    user = User.objects.get(username=config.selected_user)
    return engine.task_update(
        user=user,
        project_id=project,
        task_id=task_id,
        name=name,
        color=color,
        default=default,
    )


@command(app)
def delete(project: str, task: str):
    """Delete a task."""
    user = User.objects.get(username=config.selected_user)
    engine.task_delete(user=user, project_id=project, task_id=task)



from typing import Optional

import typer
from tea_console.console import command

from traktor.config import config
from traktor.engine import engine
from traktor.models import User, Project


app = typer.Typer(name="project", help="Project commands.")


# Make sure that the database exists and it's migrated to the latest version
app.callback()(engine.db.ensure)


@command(app, name="list", model=Project)
def projects_list():
    """List all projects."""
    user = User.objects.get(username=config.selected_user)
    return engine.project_list(user)


@command(app, name="create", model=Project)
def project_create(name: str, color: Optional[str] = None):
    """Create a project."""
    user = User.objects.get(username=config.selected_user)
    return engine.project_create(user=user, name=name, color=color)


@command(app, name="update", model=Project)
def project_update(
    project: str,
    name: Optional[str] = typer.Option(None, help="New project name."),
    color: Optional[str] = typer.Option(None, help="New project color"),
):
    """Update a project."""
    user = User.objects.get(username=config.selected_user)
    return engine.project_update(
        user=user, project_id=project, name=name, color=color
    )


@command(app, name="delete")
def project_delete(project: str):
    """Delete a project."""
    user = User.objects.get(username=config.selected_user)
    engine.project_delete(user=user, project_id=project)



from tea_django import errors


class TraktorError(errors.DjangoTeaError):
    pass


class TimerAlreadyRunning(TraktorError):
    def __init__(self, project_id: str, task_id: str):
        self.project_id = project_id
        self.task_id = task_id

        super().__init__(
            message=f"Timer is already running for {project_id}/{task_id}."
        )


class TimerIsNotRunning(TraktorError):
    def __init__(self):
        super().__init__(message="Timer is not running.")


class NoDefaultTask(TraktorError):
    def __init__(self, user: str, project_id: str):
        self.user = user
        self.project_id = project_id

        super().__init__(
            message=f"No default task found for user: {user}, "
            f"project: {project_id}."
        )



from django.dispatch import receiver
from django.db.models.signals import post_save

from traktor.models.project import Project
from traktor.models.task import Task


@receiver(post_save, sender=Project)
def create_default_task(sender, instance, created, **kwargs):
    """Create default task for every project."""
    if created:
        Task.objects.create(
            project=instance,
            name="Default",
            default=True,
            color=instance.color,
        )



import time
from typing import Optional

import typer
from tea_console.console import output

from traktor.config import config
from traktor.engine import engine
from traktor.models import User, Entry


def start(project: str, task: Optional[str] = typer.Argument(None)):
    """Start the timer."""
    user = User.objects.get(username=config.selected_user)
    return engine.timer_start(user=user, project_id=project, task_id=task)


def stop():
    """Stop the timer."""
    user = User.objects.get(username=config.selected_user)
    return engine.timer_stop(user=user)


def __output_status(user: User) -> int:
    """Output status and return number of lines printed."""
    timer = engine.timer_status(user=user)
    output(fmt=config.format, model=Entry, objs=timer)
    return 2 if timer is None else 6


def status(
    interactive: bool = typer.Option(
        False, "-i", "--interactive", help="Show status in interactive mode."
    )
):
    """See the current running timer."""
    user = User.objects.get(username=config.selected_user)
    if interactive:
        no_lines = 0
        while True:
            try:
                if no_lines > 0:
                    print("\033[F" * no_lines)
                    for _ in range(no_lines - 1):
                        print("\033[K")
                    print("\033[F" * no_lines)

                no_lines = __output_status(user=user)
                time.sleep(1)
            except KeyboardInterrupt:
                return
    else:
        __output_status(user=user)


def today():
    """See today's timers."""
    user = User.objects.get(username=config.selected_user)
    return engine.timer_today(user=user)


def report(days: int = typer.Argument(default=0, min=0)):
    """See the current running timer.

    If days is 0 that means whole history.
    """
    user = User.objects.get(username=config.selected_user)
    return engine.timer_report(user=user, days=days)



from dataclasses import dataclass

from tea import timestamp as ts
from tea_console.table import Column
from tea_django.models import VanillaModel


@dataclass
class Report(VanillaModel):
    HEADERS = VanillaModel.HEADERS + [
        Column(title="User", path="user"),
        Column(title="Project", path="project"),
        Column(title="Task", path="task"),
        Column(title="Time", path="running_time", align=Column.Align.center),
    ]

    user: str
    project: str
    task: str
    duration: int

    @property
    def key(self):
        return f"{self.project}-{self.task}"

    @property
    def running_time(self):
        return ts.humanize(self.duration)

    def to_dict(self) -> dict:
        return {
            "user": self.user,
            "project": self.project,
            "task": self.task,
            "duration": self.duration,
            "running_time": ts.humanize(self.duration),
        }



# Generated by Django 3.1 on 2020-08-28 20:19

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import traktor.models.user


class Migration(migrations.Migration):

    dependencies = [
        ("traktor", "0004_auto_20200823_1858"),
    ]

    operations = [
        migrations.RemoveField(model_name="entry", name="project",),
        migrations.AddField(
            model_name="project",
            name="user",
            field=models.ForeignKey(
                default=traktor.models.user.User.get_superuser_id,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
